#
#  APRS4R - a ruby based aprs gateway/digipeater
#  Copyright (C) 2006 by Michael Conrad <do5mc@friggleware.de>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
#

require 'aprs4r/APRS4RLogger'

require 'aprs4r/Plugin'
require 'aprs4r/APRSCall'
require 'aprs4r/BeaconPluginConfiguration'
require 'aprs4r/SocketManager'


module APRS4R

  class BeaconPlugin < Plugin

    @logger = APRS4RLogger.get_logger( "BeaconPlugin")

    attr_reader :device, :period, :offset, :message
    
    def initialize( configuration)
      logger.info( "initialize( configuration)")

      super( configuration)

      @device = configuration.device
      @period = configuration.period
      @offset = configuration.offset
      
      @message = configuration.message if configuration.message
      @message.payload = "" if @message.payload.nil?

      return
    end


    def start
      logger.info( "start")

      if ! @enable 
        return
      end

      Thread.new { run } 

      return
    end

    
    def run
      logger.info( "run")

      sleep @offset

      while true
        message = create_beacon

        if message
          logger.log( "sending beacon on device #{@device}: #{message}")
          SocketManager.send_message( @device, message)
          
          # attention: use this call with caution
          SocketManager.send_local_message( @device, message)
        end

        sleep @period
      end
      
      return
    end


    def create_beacon
      logger.info( "create_beacon")
      
      return @message
    end

  end


end
