#
#  APRS4R - a ruby based aprs gateway/digipeater
#  Copyright (C) 2006 by Michael Conrad <do5mc@friggleware.de>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
#

require 'aprs4r/APRS4RBase'
require 'aprs4r/APRS4RLogger'


module APRS4R

  class PluginManager < APRS4RBase

    @logger = APRS4RLogger.get_logger( "PluginManager")

    @@configuration = nil
    @@plugins = nil


    def PluginManager.init( configuration)
      logger.info( "init( configuration)")
      
      @@configuration = configuration.plugins
      @@plugins = Array.new

      return
    end


    def PluginManager.start
      logger.info( "start()")

      if @@configuration.nil?
        return 
      end

      begin
      
        @@configuration.each { |name, configuration| 

          logger.debug( "configuration: #{configuration}")
          
          name = configuration.name
          type = configuration.type
          enable = configuration.enable
         
          if enable 
            logger.debug( "name: #{name}")
            logger.debug( "type: #{type}")
            logger.debug( "configuration: #{configuration}")
            
            plugin = nil
            plugin_class = "aprs4r" + "/" + type
            
            # load plugin
            begin
              logger.debug( "plugin_class: #{plugin_class}")
              require plugin_class
              
              plugin = eval( "#{type}.new( configuration)")
              
              logger.debug( "plugin: #{plugin}")
              
              add_plugin( plugin) if plugin
              
            rescue Exception  => ex
              logger.error( "Unable to load/initialize plugin #{plugin_class}: #{ex}")
            end
            
            # start plugin
            begin 
              if plugin
                plugin.start()
              end
            rescue Exception => ex
              logger.error( "Unable to start plugin #{plugin}: #{ex}")
            end

          end
        }
        
        logger.debug( "all plugins loaded")

      rescue Exception => ex
        logger.error( "Initialisation failed: #{ex}")
      end

      return
    end


    def PluginManager.add_plugin( plugin)
      logger.info( "add_plugin")
      
      @@plugins << plugin
      
      return
    end
    
    
    def PluginManager.get_plugin( type)
      logger.info( "get_plugin")
      
      @@plugins.each{ |plugin|
        return plugin if plugin.is_a?( type)
      }
      
      return nil
    end


  end


end
