#
#  APRS4R - a ruby based aprs gateway/digipeater
#  Copyright (C) 2008 by Rolf Tschumi <hb9sdb@hb9gl.ch>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
#
#  SysopPlugin (c) 05. Juli 2008  by Rolf Tschumi <hb9sdb@hb9gl.ch>
#

require 'aprs4r/APRS4RLogger'

require 'aprs4r/APRSMessage'
require 'aprs4r/Plugin'
require 'aprs4r/SysopPluginConfiguration'


module APRS4R
  
  class SysopPlugin < Plugin
    
    @logger = APRS4RLogger.get_logger( "SysopPlugin")
    
    
    def initialize( configuration)
      logger.info( "initialize( configuration)")
      
      super( configuration)
      
      @device = configuration.device
      @call = configuration.call
      @sysop = configuration.sysop
      @pw = configuration.pw
      @sysoplogin = "off" 
      @sendnr = 11

      return
    end
    
    
    def start 
      logger.info( "start")

      if ! @enable
        return
      end

      register_listener

      return
    end

    
    def register_listener
      logger.info( "register_listener")

      if @enable
        SocketManager.add_listener( @device, self)
      end

      return
    end


    def unregister_listener
      logger.info( "unregister_listener")

      if @enable
        SocketManager.remove_listener( @device, self)
      end

      return
    end


    def recv_message( name, message)
      logger.info( "recv_message( name, message)")

      if message.nil? 
        return
      end

      # handle directed querys / Sysop Funktion 
      if message.is_message? && @sysop.include?( message.source.to_s)

        
        # Anfrage mit ack besttigen
        Handleack( message)        


        if message.payload.length > 11  
          
          if message.payload[11...16] == "LOGIN" && @sysoplogin == "off"
            HandleLogin( message)
          elsif message.payload[11...message.payload.length-3] == "LOGOUT" && @sysoplogin == "on"
            HandleLogout( message)
          elsif message.payload[11...message.payload.length-3] == "SYSOP" 
            HandleSysop( message)
          elsif message.payload[11...message.payload.length-3] == "VERSION" 
            HandleVersion( message)
            
          end 
        end 
        #------------------ sysop ---------------------------------------------------------------
        
        if @sysoplogin == "on" 
          
          if message.payload[11...message.payload.length-3] == "LIST"
            HandleList( message)
          elsif message.payload[11...message.payload.length-3] == "bt1"
            Handlebt1( message)
          elsif message.payload[11...message.payload.length-3] == "bt2" 
            Handlebt2( message)
          elsif message.payload[11...message.payload.length-3] == "bt3" 
            Handlebt3( message)
            
            
          end 
          
        end
        #------------------------------------------------------------------------------------------   
        
      end # handle directed querys / Sysop Funktion
      
      return
    end
    

    def Handleack( message) 
      
      meldung = message.payload[11...message.payload.length-3]
      rufzeichen = message.payload[1...10]
      msgnr = message.payload[message.payload.length-2...message.payload.length] 
      response = APRSMessage.new
      response.source = @call
      response.destination = message.destination
      response.path = message.path
      
      
      payload = "#{Callfill(message.source.to_s)}ack#{msgnr}     "
      response.payload = payload
      
      SocketManager.send_message( @device, response)
      return
    end 
    
    
    def HandleLogin( message) 
      
      meldung = message.payload[11...message.payload.length-3]
      rufzeichen = message.payload[1...10]
      msgnr = message.payload[message.payload.length-2...message.payload.length] 
      
      if meldung.length < 6 
        
        m1 = rand( @pw.length)
        m2 = rand( @pw.length)
        m3 = rand( @pw.length)
        m4 = rand( @pw.length)
        m5 = rand( @pw.length)
        
        mch1 = @pw[m1-1].chr
        mch2 = @pw[m2-1].chr
        mch3 = @pw[m3-1].chr
        mch4 = @pw[m4-1].chr
        mch5 = @pw[m5-1].chr
        
        pwg = "#{m1} #{m2} #{m3} #{m4} #{m5} "
        
        @pchg = "#{mch1}#{mch2}#{mch3}#{mch4}#{mch5}"
        pchg1 = "#{mch1}#{mch2}#{mch3}#{mch4}#{mch5}"
        
        
        # Antwort senden    
        
        response = APRSMessage.new
        
        response.source = @call
        response.destination = message.destination
        response.path = message.path
        
        
        payload = "#{Callfill(message.source.to_s)}LOGIN 401 [#{pwg}] Please send session password #{pchg1} {#{SendNum(msgnr)}    "
        
        response.payload = payload
        
        SocketManager.send_message( @device, response)
      end 
      
      
      if meldung.length > 6 && meldung[6...meldung.length] == @pchg
        
        @sysoplogin = "on"
        
        # Antwort senden    
        response = APRSMessage.new
        
        response.source = @call
        response.destination = message.destination
        response.path = message.path
        
        
        payload = "#{Callfill(message.source.to_s)}#{message.source.to_s} LOGIN 200 Login was successful {#{SendNum(msgnr)}}    "
        
        response.payload = payload
        
        SocketManager.send_message( @device, response)
        
        
      end
      
      return
    end
    
    
    def HandleLogout( message) 
      
      @sysoplogin = "off"
      
      # Antwort senden    
      meldung = message.payload[11...message.payload.length-3]
      rufzeichen = message.payload[1...10]
      msgnr = message.payload[message.payload.length-2...message.payload.length] 
      
      
      response = APRSMessage.new
      
      response.source = @call
      response.destination = message.destination
      response.path = message.path
      
      
      payload = "#{Callfill(message.source.to_s)}LOGOUT 200 Logout was successful  {#{SendNum(msgnr)}    "
      
      response.payload = payload
      
      SocketManager.send_message( @device, response)
      
      return 
    end
    

    def HandleVersion( message)
      
      meldung = message.payload[11...message.payload.length-3]
      rufzeichen = message.payload[1...10]
      msgnr = message.payload[message.payload.length-2...message.payload.length] 
      
      
      m1 = "VERSION 200 APRS4R-#{APRS4R::APRS4RVersion} Build: #{APRS4R::APRS4RBuild}" 

     
      
      # Antwort senden    
      response = APRSMessage.new
      
      response.source = @call
      response.destination = message.destination
      response.path = message.path
      
      payload = "#{Callfill(message.source.to_s)}#{m1}  {#{SendNum(msgnr)}    "

      response.payload = payload
      
      SocketManager.send_message( @device, response)
      
      return
    end

   
   
    def HandleSysop( message)
      
      meldung = message.payload[11...message.payload.length-3]
      rufzeichen = message.payload[1...10]
      msgnr = message.payload[message.payload.length-2...message.payload.length] 
      
      logger.log( "Source: #{message.source}")

      # Antwort senden    
      response = APRSMessage.new
      
      response.source = @call
      response.destination = message.destination
      response.path = message.path
      
      s1 = @sysoplogin
      
      payload = "#{Callfill(message.source.to_s)}APRS4R Sysop-Mode: #{s1} {#{SendNum(msgnr)}    "
      # payload = "#{Callfill(message.source.to_s)}APRS4R Sysop-Mode: #{s1}"
      
      
      response.payload = payload
      
      SocketManager.send_message( @device, response)
      return
    end


    def HandleList( message) 
      
      return
    end
    
    def Handlebt1( message) 
      
      return
    end
    
    
    def Handlebt2( message) 
      
      return
    end
    
    
    def Handlebt3( message)
      
      return
    end
    
    
    def Callfill( call)
      
      if call.length > 3
        call1 = call[0...call.length]
      end
      
      if call1.length == 4 
        value = ":#{call1}     :"
      end
      
      if call1.length == 5 
        value = ":#{call1}    :"
      end
      
      if call1.length == 6 
        value = ":#{call1}   :"
      end
      
      if call1.length == 7 
        value = ":#{call1}  :"
      end
      
      if call1.length == 8 
        value = ":#{call1} :"
      end
      
      if call1.length == 9 
        value = ":#{call1}:"
      end
      
      
      return value
    end
    
    
    def SendNum( msgnr)
      
      @sendnr += 1
      if @sendnr > 255 
        @sendnr = 0 
      end
      
      value = @sendnr 
      
      return value
    end
    
  end
  
end
